@@ -1,6 +1,6 @@
 #' Create a ggplot layer appropriate to a particular data type
 #'
-#' `autolayer` uses ggplot2 to draw a particular layer for an object of a
+#' `autolayer()` uses ggplot2 to draw a particular layer for an object of a
 #' particular class in a single command. This defines the S3 generic that
 #' other classes and packages can extend.
 #'
@@ -15,9 +15,5 @@ autolayer <- function(object, ...) {
 
 #' @export
 autolayer.default <- function(object, ...) {
-  abort(glue(
-    "Objects of type ",
-    glue_collapse(class(object), "/"),
-    " not supported by autolayer."
-  ))
+  cli::cli_abort("No autolayer method avialable for {.cls {class(object)[1]}} objects")
 }
