@@ -59,32 +59,58 @@ Coord <- ggproto("Coord",
 
   aspect = function(ranges) NULL,
 
-  labels = function(labels, panel_params) labels,
+  labels = function(self, labels, panel_params) {
+    # If panel params contains guides information, use it.
+    # Otherwise use the labels as is, for backward-compatibility
+    if (is.null(panel_params$guides)) {
+      return(labels)
+    }
+
+    positions_x <- c("top", "bottom")
+    positions_y <- c("left", "right")
+
+    list(
+      x = lapply(c(1, 2), function(i) {
+        panel_guide_label(
+          panel_params$guides,
+          position = positions_x[[i]],
+          default_label = labels$x[[i]]
+        )
+      }),
+      y = lapply(c(1, 2), function(i) {
+        panel_guide_label(
+          panel_params$guides,
+          position = positions_y[[i]],
+          default_label = labels$y[[i]]
+        )
+      })
+    )
+  },
 
   render_fg = function(panel_params, theme) element_render(theme, "panel.border"),
 
-  render_bg = function(panel_params, theme) {
-    abort("Not implemented")
+  render_bg = function(self, panel_params, theme) {
+    cli::cli_abort("{.fn {snake_class(self)}} has not implemented a {.fn render_bg} method")
   },
 
-  render_axis_h = function(panel_params, theme) {
-    abort("Not implemented")
+  render_axis_h = function(self, panel_params, theme) {
+    cli::cli_abort("{.fn {snake_class(self)}} has not implemented a {.fn render_axis_h} method")
   },
 
-  render_axis_v = function(panel_params, theme) {
-    abort("Not implemented")
+  render_axis_v = function(self, panel_params, theme) {
+    cli::cli_abort("{.fn {snake_class(self)}} has not implemented a {.fn render_axis_v} method")
   },
 
   # transform range given in transformed coordinates
   # back into range in given in (possibly scale-transformed)
   # data coordinates
   backtransform_range = function(self, panel_params) {
-    abort("Not implemented")
+    cli::cli_abort("{.fn {snake_class(self)}} has not implemented a {.fn backtransform_range} method")
   },
 
   # return range stored in panel_params
-  range = function(panel_params) {
-    abort("Not implemented")
+  range = function(self, panel_params) {
+    cli::cli_abort("{.fn {snake_class(self)}} has not implemented a {.fn range} method")
   },
 
   setup_panel_params = function(scale_x, scale_y, params = list()) {
@@ -92,10 +118,59 @@ Coord <- ggproto("Coord",
   },
 
   setup_panel_guides = function(self, panel_params, guides, params = list()) {
+    aesthetics <- c("x", "y", "x.sec", "y.sec")
+    names(aesthetics) <- aesthetics
+
+    # If the panel_params doesn't contain the scale, do not use a guide for that aesthetic
+    idx <- vapply(aesthetics, function(aesthetic) {
+      scale <- panel_params[[aesthetic]]
+      !is.null(scale) && inherits(scale, "ViewScale")
+    }, logical(1L))
+    aesthetics <- aesthetics[idx]
+
+    # resolve the specified guide from the scale and/or guides
+    guides <- lapply(aesthetics, function(aesthetic) {
+      resolve_guide(
+        aesthetic,
+        panel_params[[aesthetic]],
+        guides,
+        default = guide_axis(),
+        null = guide_none()
+      )
+    })
+
+    # resolve the guide definition as a "guide" S3
+    guides <- lapply(guides, validate_guide)
+
+    # if there is a "position" specification in the scale, pass this on to the guide
+    # ideally, this should be specified in the guide
+    guides <- lapply(aesthetics, function(aesthetic) {
+      guide <- guides[[aesthetic]]
+      scale <- panel_params[[aesthetic]]
+      # position could be NULL here for an empty scale
+      guide$position <- guide$position %|W|% scale$position
+      guide
+    })
+
+    panel_params$guides <- guides
     panel_params
   },
 
   train_panel_guides = function(self, panel_params, layers, default_mapping, params = list()) {
+    aesthetics <- c("x", "y", "x.sec", "y.sec")
+    names(aesthetics) <- aesthetics
+    # If the panel_params doesn't contain the scale, there's no guide for the aesthetic
+    aesthetics <- intersect(aesthetics, names(panel_params$guides))
+
+    panel_params$guides <- lapply(aesthetics, function(aesthetic) {
+      axis <- substr(aesthetic, 1, 1)
+      guide <- panel_params$guides[[aesthetic]]
+      guide <- guide_train(guide, panel_params[[aesthetic]])
+      guide <- guide_transform(guide, self, panel_params)
+      guide <- guide_geom(guide, layers, default_mapping)
+      guide
+    })
+
     panel_params
   },
 
