@@ -34,7 +34,7 @@
 #' @inheritParams geom_point
 #' @param stackdir which direction to stack the dots. "up" (default),
 #'   "down", "center", "centerwhole" (centered, but with dots aligned)
-#' @param stackratio how close to stack the dots. Default is 1, where dots just
+#' @param stackratio how close to stack the dots. Default is 1, where dots
 #'   just touch. Use smaller values for closer, overlapping dots.
 #' @param dotsize The diameter of the dots relative to `binwidth`, default 1.
 #' @param stackgroups should dots be stacked across groups? This has the effect
@@ -60,8 +60,11 @@
 #' @references Wilkinson, L. (1999) Dot plots. The American Statistician,
 #'    53(3), 276-281.
 #' @examples
-#' ggplot(mtcars, aes(x = mpg)) + geom_dotplot()
-#' ggplot(mtcars, aes(x = mpg)) + geom_dotplot(binwidth = 1.5)
+#' ggplot(mtcars, aes(x = mpg)) +
+#'   geom_dotplot()
+#'
+#' ggplot(mtcars, aes(x = mpg)) +
+#'   geom_dotplot(binwidth = 1.5)
 #'
 #' # Use fixed-width bins
 #' ggplot(mtcars, aes(x = mpg)) +
@@ -70,6 +73,7 @@
 #' # Some other stacking methods
 #' ggplot(mtcars, aes(x = mpg)) +
 #'   geom_dotplot(binwidth = 1.5, stackdir = "center")
+#'
 #' ggplot(mtcars, aes(x = mpg)) +
 #'   geom_dotplot(binwidth = 1.5, stackdir = "centerwhole")
 #'
@@ -78,10 +82,16 @@
 #'   scale_y_continuous(NULL, breaks = NULL)
 #'
 #' # Overlap dots vertically
-#' ggplot(mtcars, aes(x = mpg)) + geom_dotplot(binwidth = 1.5, stackratio = .7)
+#' ggplot(mtcars, aes(x = mpg)) +
+#'   geom_dotplot(binwidth = 1.5, stackratio = .7)
 #'
 #' # Expand dot diameter
-#' ggplot(mtcars, aes(x = mpg)) + geom_dotplot(binwidth = 1.5, dotsize = 1.25)
+#' ggplot(mtcars, aes(x = mpg)) +
+#'   geom_dotplot(binwidth = 1.5, dotsize = 1.25)
+#'
+#' # Change dot fill colour, stroke width
+#' ggplot(mtcars, aes(x = mpg)) +
+#'   geom_dotplot(binwidth = 1.5, fill = "white", stroke = 2)
 #'
 #' \donttest{
 #' # Examples with stacking along y axis instead of x
@@ -134,11 +144,15 @@ geom_dotplot <- function(mapping = NULL, data = NULL,
   # because == will fail if object is position_stack() or position_dodge()
   if (!is.null(position) &&
       (identical(position, "stack") || (inherits(position, "PositionStack"))))
-    message("position=\"stack\" doesn't work properly with geom_dotplot. Use stackgroups=TRUE instead.")
+    cli::cli_inform("{.code position = \"stack\"} doesn't work properly with {.fn geom_dotplot}. Use {.code stackgroups = TRUE} instead.")
 
   if (stackgroups && method == "dotdensity" && binpositions == "bygroup")
-    message('geom_dotplot called with stackgroups=TRUE and method="dotdensity". You probably want to set binpositions="all"')
+    cli::cli_inform(c(
+      '{.fn geom_dotplot} called with {.code stackgroups = TRUE} and {.code method = "dotdensity"}.",
+      i = "Do you want {.code binpositions = "all"} instead?'
+    ))
 
+  stackdir <- arg_match0(stackdir, c("up", "down", "center", "centerwhole"), "stackdir")
   layer(
     data = data,
     mapping = mapping,
@@ -148,7 +162,7 @@ geom_dotplot <- function(mapping = NULL, data = NULL,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
     # Need to make sure that the binaxis goes to both the stat and the geom
-    params = list(
+    params = list2(
       binaxis = binaxis,
       binwidth = binwidth,
       binpositions = binpositions,
@@ -175,7 +189,7 @@ GeomDotplot <- ggproto("GeomDotplot", Geom,
   required_aes = c("x", "y"),
   non_missing_aes = c("size", "shape"),
 
-  default_aes = aes(colour = "black", fill = "black", alpha = NA),
+  default_aes = aes(colour = "black", fill = "black", alpha = NA, stroke = 1, linetype = "solid"),
 
   setup_data = function(data, params) {
     data$width <- data$width %||%
@@ -200,23 +214,27 @@ GeomDotplot <- ggproto("GeomDotplot", Geom,
       stackaxismax <- .5
     }
 
-
     # Fill the bins: at a given x (or y), if count=3, make 3 entries at that x
     data <- data[rep(1:nrow(data), data$count), ]
 
     # Next part will set the position of each dot within each stack
     # If stackgroups=TRUE, split only on x (or y) and panel; if not stacking, also split by group
     plyvars <- params$binaxis %||% "x"
+    stackaxis <- setdiff(c("x", "y"), plyvars)
     plyvars <- c(plyvars, "PANEL")
     if (is.null(params$stackgroups) || !params$stackgroups)
       plyvars <- c(plyvars, "group")
 
+    if (stackaxis == "x") {
+      plyvars <- c(plyvars, "x")
+    }
+
     # Within each x, or x+group, set countidx=1,2,3, and set stackpos according to stack function
-    data <- plyr::ddply(data, plyvars, function(xx) {
-            xx$countidx <- 1:nrow(xx)
-            xx$stackpos <- stackdots(xx$countidx)
-            xx
-          })
+    data <- dapply(data, plyvars, function(xx) {
+      xx$countidx <- 1:nrow(xx)
+      xx$stackpos <- stackdots(xx$countidx)
+      xx
+    })
 
 
     # Set the bounding boxes for the dots
@@ -237,7 +255,7 @@ GeomDotplot <- ggproto("GeomDotplot", Geom,
       # works. They're just set to the standard x +- width/2 so that dot clusters
       # can be dodged like other geoms.
       # After position code is rewritten, each dot should have its own bounding box.
-      data <- plyr::ddply(data, c("group", "PANEL"), transform,
+      data <- dapply(data, c("group", "PANEL"), transform,
             ymin = min(y) - binwidth[1] / 2,
             ymax = max(y) + binwidth[1] / 2)
 
@@ -249,11 +267,11 @@ GeomDotplot <- ggproto("GeomDotplot", Geom,
   },
 
 
-  draw_group = function(data, panel_params, coord, na.rm = FALSE,
+  draw_group = function(data, panel_params, coord, lineend = "butt", na.rm = FALSE,
                         binaxis = "x", stackdir = "up", stackratio = 1,
                         dotsize = 1, stackgroups = FALSE) {
     if (!coord$is_linear()) {
-      warning("geom_dotplot does not work properly with non-linear coordinates.")
+      cli::cli_warn("{.fn geom_dotplot} does not work properly with non-linear coordinates.")
     }
 
     tdata <- coord$transform(data, panel_params)
@@ -273,10 +291,12 @@ GeomDotplot <- ggproto("GeomDotplot", Geom,
 
     ggname("geom_dotplot",
       dotstackGrob(stackaxis = stackaxis, x = tdata$x, y = tdata$y, dotdia = dotdianpc,
-                  stackposition = tdata$stackpos, stackratio = stackratio,
+                  stackposition = tdata$stackpos, stackdir = stackdir, stackratio = stackratio,
                   default.units = "npc",
                   gp = gpar(col = alpha(tdata$colour, tdata$alpha),
-                            fill = alpha(tdata$fill, tdata$alpha)))
+                            fill = alpha(tdata$fill, tdata$alpha),
+                            lwd = tdata$stroke, lty = tdata$linetype,
+                            lineend = lineend))
     )
   },
 
