@@ -30,13 +30,17 @@ draw_key_point <- function(data, params, size) {
     data$shape <- translate_shape_string(data$shape)
   }
 
+  # NULL means the default stroke size, and NA means no stroke.
+  stroke_size <- data$stroke %||% 0.5
+  stroke_size[is.na(stroke_size)] <- 0
+
   pointsGrob(0.5, 0.5,
     pch = data$shape,
     gp = gpar(
       col = alpha(data$colour %||% "black", data$alpha),
       fill = alpha(data$fill %||% "black", data$alpha),
-      fontsize = (data$size %||% 1.5) * .pt + (data$stroke %||% 0.5) * .stroke / 2,
-      lwd = (data$stroke %||% 0.5) * .stroke / 2
+      fontsize = (data$size %||% 1.5) * .pt + stroke_size * .stroke / 2,
+      lwd = stroke_size * .stroke / 2
     )
   )
 }
@@ -47,9 +51,9 @@ draw_key_abline <- function(data, params, size) {
   segmentsGrob(0, 0, 1, 1,
     gp = gpar(
       col = alpha(data$colour %||% data$fill %||% "black", data$alpha),
-      lwd = (data$size %||% 0.5) * .pt,
+      lwd = (data$linewidth %||% 0.5) * .pt,
       lty = data$linetype %||% 1,
-      lineend = "butt"
+      lineend = params$lineend %||% "butt"
     )
   )
 }
@@ -66,11 +70,11 @@ draw_key_rect <- function(data, params, size) {
 #' @export
 #' @rdname draw_key
 draw_key_polygon <- function(data, params, size) {
-  if (is.null(data$size)) {
-    data$size <- 0.5
+  if (is.null(data$linewidth)) {
+    data$linewidth <- 0.5
   }
 
-  lwd <- min(data$size, min(size) / 4)
+  lwd <- min(data$linewidth, min(size) / 4)
 
   rectGrob(
     width = unit(1, "npc") - unit(lwd, "mm"),
@@ -81,9 +85,7 @@ draw_key_polygon <- function(data, params, size) {
       lty = data$linetype %||% 1,
       lwd = lwd * .pt,
       linejoin = params$linejoin %||% "mitre",
-      # `lineend` is a workaround for Windows and intentionally kept unexposed
-      # as an argument. (c.f. https://github.com/tidyverse/ggplot2/issues/3037#issuecomment-457504667)
-      lineend = if (identical(params$linejoin, "round")) "round" else "square"
+      lineend = params$lineend %||% "butt"
   ))
 }
 
@@ -104,9 +106,12 @@ draw_key_boxplot <- function(data, params, size) {
     gp = gpar(
       col = data$colour %||% "grey20",
       fill = alpha(data$fill %||% "white", data$alpha),
-      lwd = (data$size %||% 0.5) * .pt,
-      lty = data$linetype %||% 1
-    )
+      lwd = (data$linewidth %||% 0.5) * .pt,
+      lty = data$linetype %||% 1,
+      lineend = params$lineend %||% "butt",
+      linejoin = params$linejoin %||% "mitre"
+    ),
+    vp = if (isTRUE(params$flipped_aes)) viewport(angle = -90)
   )
 }
 
@@ -119,9 +124,12 @@ draw_key_crossbar <- function(data, params, size) {
     gp = gpar(
       col = data$colour %||% "grey20",
       fill = alpha(data$fill %||% "white", data$alpha),
-      lwd = (data$size %||% 0.5) * .pt,
-      lty = data$linetype %||% 1
-    )
+      lwd = (data$linewidth %||% 0.5) * .pt,
+      lty = data$linetype %||% 1,
+      lineend = params$lineend %||% "butt",
+      linejoin = params$linejoin %||% "mitre"
+    ),
+    vp = if (isTRUE(params$flipped_aes)) viewport(angle = -90)
   )
 }
 
@@ -137,9 +145,11 @@ draw_key_path <- function(data, params, size) {
   segmentsGrob(0.1, 0.5, 0.9, 0.5,
     gp = gpar(
       col = alpha(data$colour %||% data$fill %||% "black", data$alpha),
-      lwd = (data$size %||% 0.5) * .pt,
+      fill = alpha(params$arrow.fill %||% data$colour
+                   %||% data$fill %||% "black", data$alpha),
+      lwd = (data$linewidth %||% 0.5) * .pt,
       lty = data$linetype %||% 1,
-      lineend = "butt"
+      lineend = params$lineend %||% "butt"
     ),
     arrow = params$arrow
   )
@@ -151,9 +161,9 @@ draw_key_vpath <- function(data, params, size) {
   segmentsGrob(0.5, 0.1, 0.5, 0.9,
     gp = gpar(
       col = alpha(data$colour %||% data$fill %||% "black", data$alpha),
-      lwd = (data$size %||% 0.5) * .pt,
+      lwd = (data$linewidth %||% 0.5) * .pt,
       lty = data$linetype %||% 1,
-      lineend = "butt"
+      lineend = params$lineend %||% "butt"
     ),
     arrow = params$arrow
   )
@@ -166,16 +176,28 @@ draw_key_dotplot <- function(data, params, size) {
     pch = 21,
     gp = gpar(
       col = alpha(data$colour %||% "black", data$alpha),
-      fill = alpha(data$fill %||% "black", data$alpha)
+      fill = alpha(data$fill %||% "black", data$alpha),
+      lty = data$linetype %||% 1,
+      lineend = params$lineend %||% "butt"
     )
   )
 }
 
+#' @export
+#' @rdname draw_key
+draw_key_linerange <- function(data, params, size) {
+  if (isTRUE(params$flipped_aes)) {
+    draw_key_path(data, params, size)
+  } else {
+    draw_key_vpath(data, params, size)
+  }
+}
+
 #' @export
 #' @rdname draw_key
 draw_key_pointrange <- function(data, params, size) {
   grobTree(
-    draw_key_vpath(data, params, size),
+    draw_key_linerange(data, params, size),
     draw_key_point(transform(data, size = (data$size %||% 1.5) * 4), params)
   )
 }
@@ -188,7 +210,7 @@ draw_key_smooth <- function(data, params, size) {
 
   grobTree(
     if (isTRUE(params$se)) rectGrob(gp = gpar(col = NA, fill = data$fill)),
-    draw_key_path(data, params)
+    draw_key_path(data, params, size)
   )
 }
 
@@ -223,9 +245,9 @@ draw_key_vline <- function(data, params, size) {
   segmentsGrob(0.5, 0, 0.5, 1,
     gp = gpar(
       col = alpha(data$colour %||% data$fill %||% "black", data$alpha),
-      lwd = (data$size %||% 0.5) * .pt,
+      lwd = (data$linewidth %||% 0.5) * .pt,
       lty = data$linetype %||% 1,
-      lineend = "butt"
+      lineend = params$lineend %||% "butt"
     )
   )
 }
@@ -244,9 +266,10 @@ draw_key_timeseries <- function(data, params, size) {
     y = c(0.1, 0.6, 0.4, 0.9),
     gp = gpar(
       col = alpha(data$colour %||% data$fill %||% "black", data$alpha),
-      lwd = (data$size %||% 0.5) * .pt,
+      lwd = (data$linewidth %||% 0.5) * .pt,
       lty = data$linetype %||% 1,
-      lineend = "butt"
+      lineend = params$lineend %||% "butt",
+      linejoin = params$linejoin %||% "round"
     )
   )
 }
