@@ -21,6 +21,9 @@
 #' @keywords internal
 #' @export
 ggplot_build <- function(plot) {
+  # Attaching the plot env to be fetched by deprecations etc.
+  attach_plot_env(plot$plot_env)
+
   UseMethod('ggplot_build')
 }
 
@@ -32,22 +35,14 @@ ggplot_build.ggplot <- function(plot) {
   }
 
   layers <- plot$layers
-  layer_data <- lapply(layers, function(y) y$layer_data(plot$data))
+  data <- rep(list(NULL), length(layers))
 
   scales <- plot$scales
-  # Apply function to layer and matching data
-  by_layer <- function(f) {
-    out <- vector("list", length(data))
-    for (i in seq_along(data)) {
-      out[[i]] <- f(l = layers[[i]], d = data[[i]])
-    }
-    out
-  }
 
   # Allow all layers to make any final adjustments based
   # on raw input data and plot info
-  data <- layer_data
-  data <- by_layer(function(l, d) l$setup_layer(d, plot))
+  data <- by_layer(function(l, d) l$layer_data(plot$data), layers, data, "computing layer data")
+  data <- by_layer(function(l, d) l$setup_layer(d, plot), layers, data, "setting up layer")
 
   # Initialise panels, add extra data for margins & missing faceting
   # variables, and add on a PANEL variable to data
@@ -55,7 +50,7 @@ ggplot_build.ggplot <- function(plot) {
   data <- layout$setup(data, plot$data, plot$plot_env)
 
   # Compute aesthetics to produce data with generalised variable names
-  data <- by_layer(function(l, d) l$compute_aesthetics(d, plot))
+  data <- by_layer(function(l, d) l$compute_aesthetics(d, plot), layers, data, "computing aesthetics")
 
   # Transform all scales
   data <- lapply(data, scales_transform_df, scales = scales)
@@ -69,17 +64,17 @@ ggplot_build.ggplot <- function(plot) {
   data <- layout$map_position(data)
 
   # Apply and map statistics
-  data <- by_layer(function(l, d) l$compute_statistic(d, layout))
-  data <- by_layer(function(l, d) l$map_statistic(d, plot))
+  data <- by_layer(function(l, d) l$compute_statistic(d, layout), layers, data, "computing stat")
+  data <- by_layer(function(l, d) l$map_statistic(d, plot), layers, data, "mapping stat to aesthetics")
 
   # Make sure missing (but required) aesthetics are added
   scales_add_missing(plot, c("x", "y"), plot$plot_env)
 
   # Reparameterise geoms from (e.g.) y and width to ymin and ymax
-  data <- by_layer(function(l, d) l$compute_geom_1(d))
+  data <- by_layer(function(l, d) l$compute_geom_1(d), layers, data, "setting up geom")
 
   # Apply position adjustments
-  data <- by_layer(function(l, d) l$compute_position(d, layout))
+  data <- by_layer(function(l, d) l$compute_position(d, layout), layers, data, "computing position")
 
   # Reset position scales, then re-train and map.  This ensures that facets
   # have control over the range of a plot: is it generated from what is
@@ -97,14 +92,17 @@ ggplot_build.ggplot <- function(plot) {
   }
 
   # Fill in defaults etc.
-  data <- by_layer(function(l, d) l$compute_geom_2(d))
+  data <- by_layer(function(l, d) l$compute_geom_2(d), layers, data, "setting up geom aesthetics")
 
   # Let layer stat have a final say before rendering
-  data <- by_layer(function(l, d) l$finish_statistics(d))
+  data <- by_layer(function(l, d) l$finish_statistics(d), layers, data, "finishing layer stat")
 
   # Let Layout modify data before rendering
   data <- layout$finish_data(data)
 
+  # Consolidate alt-text
+  plot$labels$alt <- get_alt_text(plot)
+
   structure(
     list(data = data, layout = layout, plot = plot),
     class = "ggplot_built"
@@ -155,6 +153,9 @@ layer_grob <- function(plot, i = 1L) {
 #' @param data plot data generated by [ggplot_build()]
 #' @export
 ggplot_gtable <- function(data) {
+  # Attaching the plot env to be fetched by deprecations etc.
+  attach_plot_env(data$plot$plot_env)
+
   UseMethod('ggplot_gtable')
 }
 
@@ -165,7 +166,7 @@ ggplot_gtable.ggplot_built <- function(data) {
   data <- data$data
   theme <- plot_theme(plot)
 
-  geom_grobs <- Map(function(l, d) l$draw_geom(d, layout), plot$layers, data)
+  geom_grobs <- by_layer(function(l, d) l$draw_geom(d, layout), plot$layers, data, "converting geom to grob")
   layout$setup_panel_guides(plot$guides, plot$layers, plot$mapping)
   plot_table <- layout$render(geom_grobs, data, theme, plot$labels)
 
@@ -199,13 +200,28 @@ ggplot_gtable.ggplot_built <- function(data) {
       ypos <- theme$legend.position[2]
 
       # x and y are specified via theme$legend.position (i.e., coords)
-      legend_box <- editGrob(legend_box,
-        vp = viewport(x = xpos, y = ypos, just = c(xjust, yjust),
-          height = legend_height, width = legend_width))
+      legend_box <- editGrob(
+        legend_box,
+        vp = viewport(
+          x = xpos,
+          y = ypos,
+          just = c(xjust, yjust),
+          height = legend_height,
+          width = legend_width
+        )
+      )
     } else {
       # x and y are adjusted using justification of legend box (i.e., theme$legend.justification)
-      legend_box <- editGrob(legend_box,
-        vp = viewport(x = xjust, y = yjust, just = c(xjust, yjust)))
+      legend_box <- editGrob(
+        legend_box,
+        vp = viewport(
+          x = xjust,
+          y = yjust,
+          just = c(xjust, yjust),
+          height = legend_height,
+          width = legend_width
+        )
+      )
       legend_box <- gtable_add_rows(legend_box, unit(yjust, 'null'))
       legend_box <- gtable_add_rows(legend_box, unit(1 - yjust, 'null'), 0)
       legend_box <- gtable_add_cols(legend_box, unit(xjust, 'null'), 0)
@@ -268,11 +284,11 @@ ggplot_gtable.ggplot_built <- function(data) {
   #   "plot" means align to the entire plot (except margins and tag)
   title_pos <- theme$plot.title.position %||% "panel"
   if (!(title_pos %in% c("panel", "plot"))) {
-    abort('plot.title.position should be either "panel" or "plot".')
+    cli::cli_abort('{.var plot.title.position} should be either {.val "panel"} or {.val "plot"}.')
   }
   caption_pos <- theme$plot.caption.position %||% "panel"
   if (!(caption_pos %in% c("panel", "plot"))) {
-    abort('plot.caption.position should be either "panel" or "plot".')
+    cli::cli_abort('{.var plot.caption.position} should be either {.val "panel"} or {.val "plot"}.')
   }
 
   pans <- plot_table$layout[grepl("^panel", plot_table$layout$name), , drop = FALSE]
@@ -314,8 +330,7 @@ ggplot_gtable.ggplot_built <- function(data) {
                  "bottom", "bottomright")
 
   if (!(tag_pos == "manual" || tag_pos %in% valid_pos)) {
-    abort(glue("plot.tag.position should be a coordinate or one of ",
-         glue_collapse(valid_pos, ', ', last = " or ")))
+    cli::cli_abort("{.arg plot.tag.position} should be a coordinate or one of {.or {.val {valid_pos}}}")
   }
 
   if (tag_pos == "manual") {
@@ -387,6 +402,10 @@ ggplot_gtable.ggplot_built <- function(data) {
     plot_table$layout <- plot_table$layout[c(nrow(plot_table$layout), 1:(nrow(plot_table$layout) - 1)),]
     plot_table$grobs <- plot_table$grobs[c(nrow(plot_table$layout), 1:(nrow(plot_table$layout) - 1))]
   }
+
+  # add alt-text as attribute
+  attr(plot_table, "alt-label") <- plot$labels$alt
+
   plot_table
 }
 
@@ -398,3 +417,18 @@ ggplot_gtable.ggplot_built <- function(data) {
 ggplotGrob <- function(x) {
   ggplot_gtable(ggplot_build(x))
 }
+
+# Apply function to layer and matching data
+by_layer <- function(f, layers, data, step = NULL) {
+  ordinal <- label_ordinal()
+  out <- vector("list", length(data))
+  try_fetch(
+    for (i in seq_along(data)) {
+      out[[i]] <- f(l = layers[[i]], d = data[[i]])
+    },
+    error = function(cnd) {
+      cli::cli_abort(c("Problem while {step}.", "i" = "Error occurred in the {ordinal(i)} layer."), call = layers[[i]]$constructor, parent = cnd)
+    }
+  )
+  out
+}
