@@ -34,6 +34,7 @@
 #' @seealso [sec_axis()] for how to specify secondary axes
 #' @examples
 #' last_month <- Sys.Date() - 0:29
+#' set.seed(1)
 #' df <- data.frame(
 #'   date = last_month,
 #'   price = runif(30)
@@ -66,6 +67,8 @@ scale_x_date <- function(name = waiver(),
                          date_minor_breaks = waiver(),
                          limits = NULL,
                          expand = waiver(),
+                         oob = censor,
+                         guide = waiver(),
                          position = "bottom",
                          sec.axis = waiver()) {
 
@@ -80,9 +83,10 @@ scale_x_date <- function(name = waiver(),
     date_labels = date_labels,
     minor_breaks = minor_breaks,
     date_minor_breaks = date_minor_breaks,
-    guide = "none",
+    guide = guide,
     limits = limits,
     expand = expand,
+    oob = oob,
     position = position
   )
 
@@ -100,6 +104,8 @@ scale_y_date <- function(name = waiver(),
                          date_minor_breaks = waiver(),
                          limits = NULL,
                          expand = waiver(),
+                         oob = censor,
+                         guide = waiver(),
                          position = "left",
                          sec.axis = waiver()) {
 
@@ -114,9 +120,10 @@ scale_y_date <- function(name = waiver(),
     date_labels = date_labels,
     minor_breaks = minor_breaks,
     date_minor_breaks = date_minor_breaks,
-    guide = "none",
+    guide = guide,
     limits = limits,
     expand = expand,
+    oob = oob,
     position = position
   )
 
@@ -135,6 +142,8 @@ scale_x_datetime <- function(name = waiver(),
                              timezone = NULL,
                              limits = NULL,
                              expand = waiver(),
+                             oob = censor,
+                             guide = waiver(),
                              position = "bottom",
                              sec.axis = waiver()) {
 
@@ -150,9 +159,10 @@ scale_x_datetime <- function(name = waiver(),
     minor_breaks = minor_breaks,
     date_minor_breaks = date_minor_breaks,
     timezone = timezone,
-    guide = "none",
+    guide = guide,
     limits = limits,
     expand = expand,
+    oob = oob,
     position = position
   )
 
@@ -172,6 +182,8 @@ scale_y_datetime <- function(name = waiver(),
                              timezone = NULL,
                              limits = NULL,
                              expand = waiver(),
+                             oob = censor,
+                             guide = waiver(),
                              position = "left",
                              sec.axis = waiver()) {
 
@@ -187,9 +199,10 @@ scale_y_datetime <- function(name = waiver(),
     minor_breaks = minor_breaks,
     date_minor_breaks = date_minor_breaks,
     timezone = timezone,
-    guide = "none",
+    guide = guide,
     limits = limits,
     expand = expand,
+    oob = oob,
     position = position
   )
 
@@ -208,6 +221,7 @@ scale_x_time <- function(name = waiver(),
                          expand = waiver(),
                          oob = censor,
                          na.value = NA_real_,
+                         guide = waiver(),
                          position = "bottom",
                          sec.axis = waiver()) {
 
@@ -220,6 +234,7 @@ scale_x_time <- function(name = waiver(),
     expand = expand,
     oob = oob,
     na.value = na.value,
+    guide = guide,
     position = position,
     trans = scales::hms_trans(),
     sec.axis = sec.axis
@@ -237,6 +252,7 @@ scale_y_time <- function(name = waiver(),
                          expand = waiver(),
                          oob = censor,
                          na.value = NA_real_,
+                         guide = waiver(),
                          position = "left",
                          sec.axis = waiver()) {
 
@@ -249,6 +265,7 @@ scale_y_time <- function(name = waiver(),
     expand = expand,
     oob = oob,
     na.value = na.value,
+    guide = guide,
     position = position,
     trans = scales::hms_trans(),
     sec.axis = sec.axis
@@ -298,6 +315,11 @@ datetime_scale <- function(aesthetics, trans, palette,
     scale_class <- ScaleContinuous
   }
 
+  trans <- switch(trans,
+    date = date_trans(),
+    time = time_trans(timezone)
+  )
+
   sc <- continuous_scale(
     aesthetics,
     name,
@@ -366,6 +388,14 @@ ScaleContinuousDate <- ggproto("ScaleContinuousDate", ScaleContinuous,
   map = function(self, x, limits = self$get_limits()) {
     self$oob(x, limits)
   },
+  get_breaks = function(self, limits = self$get_limits()) {
+    breaks <- ggproto_parent(ScaleContinuous, self)$get_breaks(limits)
+    if (is.null(breaks)) {
+      return(NULL)
+    }
+    breaks <- floor(breaks)
+    breaks[breaks >= limits[1] & breaks <= limits[2]]
+  },
   break_info = function(self, range = NULL) {
     breaks <- ggproto_parent(ScaleContinuous, self)$break_info(range)
     if (!(is.waive(self$secondary.axis) || self$secondary.axis$empty())) {
