@@ -18,6 +18,13 @@
 #'  \item{`nlevel`}{Height of contour, scaled to maximum of 1.}
 #'  \item{`piece`}{Contour piece (an integer).}
 #' }
+#'
+#' @section Dropped variables:
+#' \describe{
+#'   \item{`z`}{After contouring, the z values of individual data points are no longer available.}
+#' }
+#'
+#'
 #' @rdname geom_contour
 stat_contour <- function(mapping = NULL, data = NULL,
                          geom = "contour", position = "identity",
@@ -36,7 +43,7 @@ stat_contour <- function(mapping = NULL, data = NULL,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(
+    params = list2(
       bins = bins,
       binwidth = binwidth,
       breaks = breaks,
@@ -65,7 +72,7 @@ stat_contour_filled <- function(mapping = NULL, data = NULL,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(
+    params = list2(
       bins = bins,
       binwidth = binwidth,
       breaks = breaks,
@@ -83,6 +90,7 @@ StatContour <- ggproto("StatContour", Stat,
 
   required_aes = c("x", "y", "z"),
   default_aes = aes(order = after_stat(level)),
+  dropped_aes = "z", # z gets dropped during statistical transformation
 
   setup_params = function(data, params) {
     params$z.range <- range(data$z, na.rm = TRUE, finite = TRUE)
@@ -112,6 +120,7 @@ StatContourFilled <- ggproto("StatContourFilled", Stat,
 
   required_aes = c("x", "y", "z"),
   default_aes = aes(order = after_stat(level), fill = after_stat(level)),
+  dropped_aes = "z", # z gets dropped during statistical transformation
 
   setup_params = function(data, params) {
     params$z.range <- range(data$z, na.rm = TRUE, finite = TRUE)
@@ -144,10 +153,17 @@ StatContourFilled <- ggproto("StatContourFilled", Stat,
 #' @noRd
 #'
 contour_breaks <- function(z_range, bins = NULL, binwidth = NULL, breaks = NULL) {
-  if (!is.null(breaks)) {
+  breaks <- allow_lambda(breaks)
+
+  if (is.numeric(breaks)) {
     return(breaks)
   }
 
+  breaks_fun <- fullseq
+  if (is.function(breaks)) {
+    breaks_fun <- breaks
+  }
+
   # If no parameters set, use pretty bins
   if (is.null(bins) && is.null(binwidth)) {
     breaks <- pretty(z_range, 10)
@@ -167,20 +183,20 @@ contour_breaks <- function(z_range, bins = NULL, binwidth = NULL, breaks = NULL)
     }
 
     binwidth <- diff(z_range) / (bins - 1)
-    breaks <- fullseq(z_range, binwidth)
+    breaks <- breaks_fun(z_range, binwidth)
 
     # Sometimes the above sequence yields one bin too few.
     # If this happens, try again.
     if (length(breaks) < bins + 1) {
       binwidth <- diff(z_range) / bins
-      breaks <- fullseq(z_range, binwidth)
+      breaks <- breaks_fun(z_range, binwidth)
     }
 
     return(breaks)
   }
 
   # if we haven't returned yet, compute breaks from binwidth
-  fullseq(z_range, binwidth)
+  breaks_fun(z_range, binwidth)
 }
 
 #' Compute isoband objects
@@ -194,8 +210,8 @@ contour_breaks <- function(z_range, bins = NULL, binwidth = NULL, breaks = NULL)
 #'
 xyz_to_isolines <- function(data, breaks) {
   isoband::isolines(
-    x = sort(unique(data$x)),
-    y = sort(unique(data$y)),
+    x = sort(unique0(data$x)),
+    y = sort(unique0(data$y)),
     z = isoband_z_matrix(data),
     levels = breaks
   )
@@ -203,8 +219,8 @@ xyz_to_isolines <- function(data, breaks) {
 
 xyz_to_isobands <- function(data, breaks) {
   isoband::isobands(
-    x = sort(unique(data$x)),
-    y = sort(unique(data$y)),
+    x = sort(unique0(data$x)),
+    y = sort(unique0(data$y)),
     z = isoband_z_matrix(data),
     levels_low = breaks[-length(breaks)],
     levels_high = breaks[-1]
@@ -223,8 +239,8 @@ xyz_to_isobands <- function(data, breaks) {
 #'
 isoband_z_matrix <- function(data) {
   # Convert vector of data to raster
-  x_pos <- as.integer(factor(data$x, levels = sort(unique(data$x))))
-  y_pos <- as.integer(factor(data$y, levels = sort(unique(data$y))))
+  x_pos <- as.integer(factor(data$x, levels = sort(unique0(data$x))))
+  y_pos <- as.integer(factor(data$y, levels = sort(unique0(data$y))))
 
   nrow <- max(y_pos)
   ncol <- max(x_pos)
@@ -247,8 +263,8 @@ iso_to_path <- function(iso, group = 1) {
   lengths <- vapply(iso, function(x) length(x$x), integer(1))
 
   if (all(lengths == 0)) {
-    warn("stat_contour(): Zero contours were generated")
-    return(new_data_frame())
+    cli::cli_warn("{.fn stat_contour}: Zero contours were generated")
+    return(data_frame0())
   }
 
   levels <- names(iso)
@@ -261,15 +277,13 @@ iso_to_path <- function(iso, group = 1) {
   groups <- paste(group, sprintf("%03d", item_id), sprintf("%03d", ids), sep = "-")
   groups <- factor(groups)
 
-  new_data_frame(
-    list(
-      level = rep(levels, lengths),
-      x = xs,
-      y = ys,
-      piece = as.integer(groups),
-      group = groups
-    ),
-    n = length(xs)
+  data_frame0(
+    level = rep(levels, lengths),
+    x = xs,
+    y = ys,
+    piece = as.integer(groups),
+    group = groups,
+    .size = length(xs)
   )
 }
 
@@ -285,8 +299,8 @@ iso_to_polygon <- function(iso, group = 1) {
   lengths <- vapply(iso, function(x) length(x$x), integer(1))
 
   if (all(lengths == 0)) {
-    warn("stat_contour(): Zero contours were generated")
-    return(new_data_frame())
+    cli::cli_warn("{.fn stat_contour}: Zero contours were generated")
+    return(data_frame0())
   }
 
   levels <- names(iso)
@@ -299,16 +313,14 @@ iso_to_polygon <- function(iso, group = 1) {
   groups <- paste(group, sprintf("%03d", item_id), sep = "-")
   groups <- factor(groups)
 
-  new_data_frame(
-    list(
-      level = rep(levels, lengths),
-      x = xs,
-      y = ys,
-      piece = as.integer(groups),
-      group = groups,
-      subgroup = ids
-    ),
-    n = length(xs)
+  data_frame0(
+    level = rep(levels, lengths),
+    x = xs,
+    y = ys,
+    piece = as.integer(groups),
+    group = groups,
+    subgroup = ids,
+    .size = length(xs)
   )
 }
 
@@ -321,11 +333,17 @@ iso_to_polygon <- function(iso, group = 1) {
 #' @noRd
 #'
 pretty_isoband_levels <- function(isoband_levels, dig.lab = 3) {
-  interval_low <- gsub(":.*$", "", isoband_levels)
-  interval_high <- gsub("^[^:]*:", "", isoband_levels)
+  interval_low <- as.numeric(gsub(":.*$", "", isoband_levels))
+  interval_high <- as.numeric(gsub("^[^:]*:", "", isoband_levels))
 
-  label_low <- format(as.numeric(interval_low), digits = dig.lab, trim = TRUE)
-  label_high <- format(as.numeric(interval_high), digits = dig.lab, trim = TRUE)
+  breaks <- unique(c(interval_low, interval_high))
+
+  while(anyDuplicated(format(breaks, digits = dig.lab, trim = TRUE))) {
+    dig.lab <- dig.lab + 1
+  }
+
+  label_low <- format(interval_low, digits = dig.lab, trim = TRUE)
+  label_high <- format(interval_high, digits = dig.lab, trim = TRUE)
 
   # from the isoband::isobands() docs:
   # the intervals specifying isobands are closed at their lower boundary
