@@ -7,6 +7,11 @@
 #' The downside is that it requires more training to accurately interpret,
 #' and the underlying visual tasks are somewhat more challenging.
 #'
+#' The statistic relies on the aesthetics assignment to guess which variable to
+#' use as the input and which to use as the output. Either x or y must be provided
+#' and one of them must be unused. The ECDF will be calculated on the given aesthetic
+#' and will be output on the unused one.
+#'
 #' @inheritParams layer
 #' @inheritParams geom_point
 #' @param na.rm If `FALSE` (the default), removes missing values with
@@ -17,22 +22,25 @@
 #'   and (Inf, 1)
 #' @section Computed variables:
 #' \describe{
-#'   \item{x}{x in data}
 #'   \item{y}{cumulative density corresponding x}
 #' }
 #' @export
 #' @examples
+#' set.seed(1)
 #' df <- data.frame(
 #'   x = c(rnorm(100, 0, 3), rnorm(100, 0, 10)),
 #'   g = gl(2, 100)
 #' )
-#' ggplot(df, aes(x)) + stat_ecdf(geom = "step")
+#' ggplot(df, aes(x)) +
+#'   stat_ecdf(geom = "step")
 #'
 #' # Don't go to positive/negative infinity
-#' ggplot(df, aes(x)) + stat_ecdf(geom = "step", pad = FALSE)
+#' ggplot(df, aes(x)) +
+#'   stat_ecdf(geom = "step", pad = FALSE)
 #'
 #' # Multiple ECDFs
-#' ggplot(df, aes(x, colour = g)) + stat_ecdf()
+#' ggplot(df, aes(x, colour = g)) +
+#'   stat_ecdf()
 stat_ecdf <- function(mapping = NULL, data = NULL,
                       geom = "step", position = "identity",
                       ...,
@@ -49,7 +57,7 @@ stat_ecdf <- function(mapping = NULL, data = NULL,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(
+    params = list2(
       n = n,
       pad = pad,
       na.rm = na.rm,
@@ -64,10 +72,27 @@ stat_ecdf <- function(mapping = NULL, data = NULL,
 #' @usage NULL
 #' @export
 StatEcdf <- ggproto("StatEcdf", Stat,
-  compute_group = function(data, scales, n = NULL, pad = TRUE) {
+  required_aes = c("x|y"),
+
+  default_aes = aes(y = after_stat(y)),
+
+  setup_params = function(self, data, params) {
+    params$flipped_aes <- has_flipped_aes(data, params, main_is_orthogonal = FALSE, main_is_continuous = TRUE)
+
+    has_x <- !(is.null(data$x) && is.null(params$x))
+    has_y <- !(is.null(data$y) && is.null(params$y))
+    if (!has_x && !has_y) {
+      cli::cli_abort("{.fn {snake_class(self)}} requires an {.field x} or {.field y} aesthetic.")
+    }
+
+    params
+  },
+
+  compute_group = function(data, scales, n = NULL, pad = TRUE, flipped_aes = FALSE) {
+    data <- flip_data(data, flipped_aes)
     # If n is NULL, use raw values; otherwise interpolate
     if (is.null(n)) {
-      x <- unique(data$x)
+      x <- unique0(data$x)
     } else {
       x <- seq(min(data$x), max(data$x), length.out = n)
     }
@@ -75,13 +100,15 @@ StatEcdf <- ggproto("StatEcdf", Stat,
     if (pad) {
       x <- c(-Inf, x, Inf)
     }
-    y <- ecdf(data$x)(x)
+    data_ecdf <- ecdf(data$x)(x)
 
-    new_data_frame(list(x = x, y = y), n = length(x))
-  },
-
-  default_aes = aes(y = after_stat(y)),
-
-  required_aes = c("x")
+    df_ecdf <- data_frame0(
+      x = x,
+      y = data_ecdf,
+      .size = length(x)
+    )
+    df_ecdf$flipped_aes <- flipped_aes
+    flip_data(df_ecdf, flipped_aes)
+  }
 )
 
