@@ -28,7 +28,7 @@ stat_quantile <- function(mapping = NULL, data = NULL,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(
+    params = list2(
       quantiles = quantiles,
       formula = formula,
       method = method,
@@ -50,7 +50,7 @@ StatQuantile <- ggproto("StatQuantile", Stat,
   compute_group = function(data, scales, quantiles = c(0.25, 0.5, 0.75),
                            formula = NULL, xseq = NULL, method = "rq",
                            method.args = list(), lambda = 1, na.rm = FALSE) {
-    try_require("quantreg", "stat_quantile")
+    check_installed("quantreg", reason = "for `stat_quantile()`")
 
     if (is.null(formula)) {
       if (method == "rqss") {
@@ -64,8 +64,7 @@ StatQuantile <- ggproto("StatQuantile", Stat,
       } else {
         formula <- y ~ x
       }
-      message("Smoothing formula not specified. Using: ",
-        deparse(formula))
+      cli::cli_inform("Smoothing formula not specified. Using: {deparse(formula)}")
     }
 
     if (is.null(data$weight)) data$weight <- 1
@@ -75,7 +74,7 @@ StatQuantile <- ggproto("StatQuantile", Stat,
       xmax <- max(data$x, na.rm = TRUE)
       xseq <- seq(xmin, xmax, length.out = 100)
     }
-    grid <- new_data_frame(list(x = xseq))
+    grid <- data_frame0(x = xseq, .size = length(xseq))
 
     # if method was specified as a character string, replace with
     # the corresponding function
@@ -87,16 +86,29 @@ StatQuantile <- ggproto("StatQuantile", Stat,
       method <- match.fun(method) # allow users to supply their own methods
     }
 
-    rbind_dfs(lapply(quantiles, quant_pred, data = data, method = method,
-      formula = formula, weight = weight, grid = grid, method.args = method.args))
+    result <- lapply(
+      quantiles,
+      quant_pred,
+      data = data,
+      method = method,
+      formula = formula,
+      weight = weight,
+      grid = grid,
+      method.args = method.args
+    )
+    vec_rbind0(!!!result)
   }
 )
 
 quant_pred <- function(quantile, data, method, formula, weight, grid,
                        method.args = method.args) {
-  args <- c(list(quote(formula), data = quote(data), tau = quote(quantile),
-    weights = quote(weight)), method.args)
-  model <- do.call(method, args)
+  model <- inject(method(
+    formula,
+    data = data,
+    tau = quantile,
+    weights = weight,
+    !!!method.args
+  ))
 
   grid$y <- stats::predict(model, newdata = grid)
   grid$quantile <- quantile
