@@ -1,5 +1,3 @@
-context("scale_date")
-
 
 # Visual tests ------------------------------------------------------------
 
@@ -14,7 +12,7 @@ test_that("date scale draws correctly", {
   )
   df <- df[order(df$dx), ]
 
-  dt <- qplot(dx, price, data = df, geom = "line")
+  dt <- ggplot(df, aes(dx, price)) + geom_line()
   expect_doppelganger("dates along x, default breaks",
     dt
   )
@@ -31,7 +29,7 @@ test_that("date scale draws correctly", {
     dt + scale_x_date(labels = date_format("%W"), "week")
   )
 
-  dt <- qplot(price, dx, data = df, geom = "line")
+  dt <- ggplot(df, aes(price, dx)) + geom_line()
   expect_doppelganger("dates along y, default breaks", dt)
   expect_doppelganger("scale_y_date(breaks = date_breaks(\"2 weeks\"))",
     dt + scale_y_date(breaks = date_breaks("2 weeks"))
